/*
 Illinois Open Source License

 University of Illinois/NCSA
 Open Source License

 Copyright © 2009,    University of Illinois.  All rights reserved.

 Developed by:
 Innovative Systems Lab
 National Center for Supercomputing Applications
 http://www.ncsa.uiuc.edu/AboutUs/Directorates/ISL.html

 Permission is hereby granted, free of charge, to any person obtaining a copy of this 
 software and associated documentation files (the "Software"), to deal with the Software 
 without restriction, including without limitation the rights to use, copy, modify, merge, 
 publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
 persons to whom the Software is furnished to do so, subject to the following conditions:
 
 * Redistributions of source code must retain the above copyright notice, this list of 
   conditions and the following disclaimers.
 
 * Redistributions in binary form must reproduce the above copyright notice, this list of 
   conditions and the following disclaimers in the documentation and/or other materials 
   provided with the distribution.
 
 * Neither the names of Innovative Systems Lab and National Center for Supercomputing 
   Applications, nor the names of its contributors may be used to endorse or promote 
   products derived from this Software without specific prior written permission.
 
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
   INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE 
   LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
   OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
   DEALINGS WITH THE SOFTWARE.
 */

#ifndef MODEL_H
#define MODEL_H

#define d2r M_PI/180.0
#define r2d 180.0/M_PI
#define r2am 60.0*180.0/M_PI

#include <stdint.h>

struct spherical {
    double ra, dec; //latitude, longitude pair
};

struct cartesian {
    double x, y, z; //cartesian coordinates
    int jk; int model; //jackknife estimate form which sample is removed    
};

struct cartesian2 {
    double x[2];
    double y[2];
    double z[2]; 
    double jk[2]; 
};

#define MAXBINS 32
#define MAXNJK 16

int readdatafile(char* fname, struct cartesian *data,struct cartesian2* data_v, int npoints);
void assign_jk(struct cartesian *data, int npoints, int njk);
int doComputeWaterfall(struct cartesian *data1, int n1, struct cartesian  *data2, struct cartesian2* data2_v,int n2, int doSelf, long long data_bins[][MAXBINS], int njk, int, int);
int doComputeWaterfall_cpu(struct cartesian *data1, int n1, struct cartesian  *data2, int n2, int doSelf, long long data_bins[][MAXBINS], int nbins, double *binb, int njk);

#endif
