/*
 Illinois Open Source License

 University of Illinois/NCSA
 Open Source License

 Copyright © 2009,    University of Illinois.  All rights reserved.

 Developed by:
 Innovative Systems Lab
 National Center for Supercomputing Applications
 http://www.ncsa.uiuc.edu/AboutUs/Directorates/ISL.html

 Permission is hereby granted, free of charge, to any person obtaining a copy of this 
 software and associated documentation files (the "Software"), to deal with the Software 
 without restriction, including without limitation the rights to use, copy, modify, merge, 
 publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
 persons to whom the Software is furnished to do so, subject to the following conditions:
 
 * Redistributions of source code must retain the above copyright notice, this list of 
   conditions and the following disclaimers.
 
 * Redistributions in binary form must reproduce the above copyright notice, this list of 
   conditions and the following disclaimers in the documentation and/or other materials 
   provided with the distribution.
 
 * Neither the names of Innovative Systems Lab and National Center for Supercomputing 
   Applications, nor the names of its contributors may be used to endorse or promote 
   products derived from this Software without specific prior written permission.
 
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
   INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE 
   LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
   OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
   DEALINGS WITH THE SOFTWARE.
 */

#ifndef __SPU_MISC_H__
#define __SPU_MISC_H__


#define ALIGNMENT 16
#if 1
#define DMACHECK(ls, ea, size, tag, cmd)                                                           \
 if ( ((unsigned int)(ls) % ALIGNMENT != 0 || (unsigned int)(ea) % ALIGNMENT != 0)		\
	|| (size == 0) || (ls == 0) || (ea == 0) || (size > 16384)				\
	|| ( tag <0 || tag >31) ){ \
    spu_log("ERROR: invalid arguments\n");                                                              \
    spu_log("ERROR: line %d ls=%p, ea=%p, size=%d, tag=%d\n", __LINE__, (char*)ls, (char*)ea, (int)size, tag);       \
    _exit(1);                                                                                           \
 }

#else
#define DMACHECK(ls, ea, size, tag, cmd) 
#endif

#define DODMA(ls,ea, size, tag, cmd) \
do{                                     \
    DMACHECK(ls, ea, size, tag, cmd);    \
    spu_mfcdma32((char*)(ls), (unsigned int)(ea), size, tag, cmd); \
} while(0)                                              \



#define PRINTVF(X)		\
			    spu_log("%s =%f %f %f %f\n",  \
			    #X, \
			    spu_extract(X, 0), \
			    spu_extract(X, 1), \
			    spu_extract(X, 2), \
			    spu_extract(X, 3)); 
#define PRINTVD(X)		\
			    spu_log("%s =%f %f\n",  \
			    #X, \
			    spu_extract(X, 0), \
			    spu_extract(X, 1)); 
#define PRINTVI(X)		\
			    spu_log("%s =%d %d %d %d\n",  \
			    #X, \
			    spu_extract(X, 0), \
			    spu_extract(X, 1), \
			    spu_extract(X, 2), \
			    spu_extract(X, 3)); 
#define PRINTVS(X)		\
			    spu_log("%s =%d %d %d %d %d %d %d %d\n",  \
			    #X, \
			    spu_extract(X, 0), \
			    spu_extract(X, 1), \
			    spu_extract(X, 2), \
			    spu_extract(X, 3), \
			    spu_extract(X, 4), \
			    spu_extract(X, 5), \
			    spu_extract(X, 6), \
			    spu_extract(X, 7)); 



#define CHECKTEMPBINS() \
do { int j; \
    for (j =0; j < nbins+2; j++){ \
	if (temp_data_bins[j] < 0){ \
	    spu_log("temp_data_bins[%d] less than 0 (%lld) (line %d)\n", j, temp_data_bins[j], __LINE__); \
	    exit(1); \
	} \
    } \
} while(0)

#define CHECKTEMPBINS2() \
do { int j; \
    for (j =0; j < nbins+2; j++){ \
	if (temp_data_bins[j] < 0){ \
	    spu_log("jk=%d\n", jk);\
	    spu_log("temp_data_bins[%d] less than 0 (%lld) (line %d)\n", j, temp_data_bins[j], __LINE__); \
	    exit(1); \
	} \
    } \
} while(0)


#endif
